%% Evaluate segmentation and tracking results for the paper
% ========================================
% 
% "Joint Tracking and Segmentation of Multiple Targets".
% A. Milan, L. Leal-Taixé, K. Schindler and I. Reid.
% CVPR 2015
% 
% 
% This package contains all necessary data and scripts to produce the numbers
% shown in Tables 2-4 of the paper. Results in Tab. 5 were produced using
% Yang Bo's evaluation script.
% 
% The baseline results for cl2, DP, DCO, and H2T are also included. 
% cl2 and DP have been generated by us using the publicly available code, while
% DCO and H2T are the original results (bounding boxes), provided by the authors.
% 
% Note that the evaluation uses a slightly modified CLEAR_MOT.m where a minor bug 
% has been fixed. The original implementation is also included as CLEAR_MOT_OLD.m
% but is *not* recommended to be used.

%% Prepare
clear all
addpath(genpath('.'));

%% Segmentation
fprintf('\n* * *   SEGMENTATION   * * *\n');

% load ground truth
load data/S2L2-gt-1-30
gtFrames = [1,5,10,15,30]; % these frames are annotated with masks
gt=gt(:,:,gtFrames);

% TSP
fprintf('\n--- Temporal Super-Pixels (TSP) ---\n');
load data/TSPSeg
evalSeg(gt, resTSP);

% Greedy labeling
fprintf('\n--- Greedy ---\n');
load data/GreedySeg
evalSeg(gt, resOursBaseline);

% our result
fprintf('\n--- Our result ---\n');
load data/ourSeg
evalSeg(gt, resOurs);


%% Tracking
% load ground truth
load data/gtInfos

fprintf('\n* * *   TRACKING   * * *\n');

%% Evaluation on ground plane on six sequences
fprintf('\n* * * * * * * * * * *\n');
fprintf('S2L1, S2L2, S2L3, S1L1-2, S1L2-1, TUD-Stadtmitte\n');
fprintf('Comparison to cl2, DP, DCO\n');
allscen=[23,25,27,71,72,42];  % S2L1, S2L2, S2L3, S1L1-2, S1L2-1, TUD-Stadtmitte
evopts.eval3d=1; evopts.td=1000;

fprintf('\n--- cl2 results...\n');
load data/cl2Infos
allmets=evalDataset(c2lInfos,gtInfos,allscen,evopts,0);

fprintf('\n--- DP results...\n');
load data/DPInfos
allmets=evalDataset(dpInfos,gtInfos,allscen,evopts,0);

fprintf('\n--- DCO results...\n');
load data/dcoInfos
allmets=evalDataset(dcoInfos,gtInfos,allscen,evopts,0);

fprintf('\n--- Our results...\n');
load data/ourInfos
allmets=evalDataset(infos,gtInfos,allscen,evopts,0);


%% Evaluation on ground plane (cropped tracking area)
fprintf('\n* * * * * * * * * * *\n');
fprintf('\nComparison to CEM\n');
allscen=[23,25,27,71,72,42]; % S2L1, S2L2, S2L3, S1L1-2, S1L2-1, TUD-Stadtmitte

evopts.eval3d=1; evopts.td=1000;
fprintf('\n--- Our results...\n');
allmets=evalDataset(infos,gtInfos,allscen,evopts,1);


%
fprintf('\n* * * * * * * * * * *\n');
fprintf('Comparison to H2T...\n');
fprintf('S2L1, S2L2, S2L3, S1L1-1, S1L1-2\n');
load data/H2TInfos
allscen=[23,25,27,70,71]; % S2L1, S2L2, S2L3, S1L1-1, S1L1-2

evopts.eval3d=1; evopts.td=1000;

fprintf('\n--- H2T results...\n');
allmets=evalDataset(wenInfos,gtInfos,allscen,evopts,1);

fprintf('\n--- Our results...\n');
allmets=evalDataset(infos,gtInfos,allscen,evopts,1);


